import math
import os

class SceneExporter:
	def __init__(self):
		self.resource_collection = True
		self.show_progress = True

	def export_to_file(self, context, out_fn):
		objs = [o for o in context.selected_objects if o.type=="MESH" and not o.lod_for_parent]
		objs = [o for o in objs if (not o.compound or o.parent not in objs)]

		path, base = os.path.split(out_fn)
		base, ext = os.path.splitext(base)

		from .export_object import ObjectExporter
		object_export = ObjectExporter()

		object_prototypes = {}
		unique_objects = []
		export_names = {}
		for o in objs:
			if o.name in object_prototypes:
				continue

			clones = [o]
			if not any(s.link=="OBJECT" for s in o.material_slots):
				for u in objs:
					if u is o:
						continue
					if u.data.name!=o.data.name:
						continue
					if u.technique!=o.technique:
						continue
					if any(s.link=="OBJECT" for s in u.material_slots):
						continue

					clones.append(u)

			prefix = o.name
			for c in clones:
				while not c.name.startswith(prefix):
					pos = max(prefix.rfind(' '), prefix.rfind('.'))
					if pos<0:
						break;
					prefix = prefix[:pos]

			if prefix:
				export_names[o.name+".object"] = prefix.strip(" .")+".object"

			unique_objects.append(o)
			for c in clones:
				object_prototypes[c.name] = o

		from .util import Progress
		progress = Progress(self.show_progress and context)

		resources = {}
		self.export_scene_resources(context, unique_objects, resources, progress)
		for n, r in resources.items():
			if r.name in export_names:
				r.name = export_names[r.name]

		scene_res = self.export_scene(context, objs, progress, prototypes=object_prototypes, resources=resources)
		refs = scene_res.collect_references()

		from .datafile import Statement
		if self.resource_collection:
			keywords = { ".mat": "material",
				".mesh": "mesh",
				".object": "object",
				".tech": "technique",
				".tex2d": "texture2d" }
			with open(os.path.join(path, base+"_resources.mdc"), "w") as res_out:
				for r in refs:
					st = Statement(keywords[os.path.splitext(r.name)[1]], r.name)
					st.sub = r.statements
					st.write_to_file(res_out)
		else:
			res_dir = os.path.join(path, base+"_resources")
			if not os.path.exists(res_dir):
				os.makedirs(res_dir)
			for r in refs:
				with open(os.path.join(res_dir, r.name), "w") as res_out:
					for s in r.statements:
						s.write_to_file(res_out)

		with open(out_fn, "w") as out_file:
			for s in scene_res.statements:
				s.write_to_file(out_file)

	def export_scene_resources(self, context, objs, resources, progress):
		from .export_object import ObjectExporter
		object_export = ObjectExporter()
		object_export.single_file = False

		material_maps = {}

		for i, o in enumerate(objs):
			progress.push_task_slice(o.name, i, len(objs))
			object_export.export_object_resources(context, o, resources, progress, material_maps=material_maps)
			obj_name = o.name+".object"
			resources[obj_name] = object_export.export_object(context, o, progress, resources=resources)
			progress.pop_task()

	def export_scene(self, context, objs, progress, *, prototypes, resources):
		from .datafile import Resource, Statement
		scene_res = Resource("scene.scene")

		for o in objs:
			obj_res = resources[prototypes[o.name].name+".object"]
			st = scene_res.create_reference_statement("object", obj_res, o.name)
			# XXX Parent relationships screw up the location and rotation
			ss = Statement("transform")
			ss.sub.append(Statement("position", o.location[0], o.location[1], o.location[2]))
			if o.rotation_mode=="AXIS_ANGLE":
				angle = o.rotation_axis_angle[0]
				axis = o.rotation_axis_angle[1:]
			else:
				if o.rotation_mode=="QUATERNION":
					q = o.rotation_quaternion
				else:
					q = o.rotation_euler.to_quaternion()
				angle = q.angle
				axis = q.axis
			ss.sub.append(Statement("rotation", angle*180/math.pi, axis[0], axis[1], axis[2]))
			ss.sub.append(Statement("scale", o.scale[0], o.scale[1], o.scale[2]))
			st.sub.append(ss)
			scene_res.statements.append(st)

		progress.set_progress(1.0)

		return scene_res
